@@ -52,10 +52,6 @@
 #include "util.h"
 #include "interface.h"
 #include "selinux.h"
-extern "C" {
-#include "e_date.h"
-}
-
 
 ProcInfo::List ProcInfo::all;
 std::map<pid_t, guint64> ProcInfo::cpu_times;
@@ -70,35 +66,6 @@ ProcInfo* ProcInfo::find(pid_t pid)
 
 
 
-static gint
-sort_ints (GtkTreeModel *model, GtkTreeIter *itera, GtkTreeIter *iterb, gpointer data)
-{
-	ProcInfo *infoa = NULL, *infob = NULL;
-	const gint col = GPOINTER_TO_INT (data);
-
-	gtk_tree_model_get (model, itera, COL_POINTER, &infoa, -1);
-	gtk_tree_model_get (model, iterb, COL_POINTER, &infob, -1);
-
-	g_assert(infoa);
-	g_assert(infob);
-
-	switch (col) {
-	case COL_CPU_TIME:
-		return PROCMAN_RCMP(infoa->cpu_time_last, infob->cpu_time_last);
-
-	case COL_START_TIME:
-		return PROCMAN_CMP(infoa->start_time, infob->start_time);
-
-	case COL_CPU:
-		return PROCMAN_RCMP(infoa->pcpu, infob->pcpu);
-	default:
-		g_assert_not_reached();
-		return 0;
-	}
-}
-
-
-
 static void
 set_proctree_reorderable(ProcData *procdata)
 {
@@ -271,15 +238,15 @@ proctable_new (ProcData * const procdata)
 	model = gtk_tree_store_new (NUM_COLUMNS,
 				    G_TYPE_STRING,	/* Process Name */
 				    G_TYPE_STRING,	/* User		*/
-				    G_TYPE_STRING,	/* Status	*/
+				    G_TYPE_UINT,	/* Status	*/
 				    G_TYPE_ULONG,	/* VM Size	*/
 				    G_TYPE_ULONG,	/* Resident Memory */
 				    G_TYPE_ULONG,	/* Writable Memory */
 				    G_TYPE_ULONG,	/* Shared Memory */
 				    G_TYPE_ULONG,	/* X Server Memory */
 				    G_TYPE_UINT,	/* % CPU	*/
-				    G_TYPE_STRING,	/* CPU time	*/
-				    G_TYPE_STRING,	/* Started	*/
+				    G_TYPE_UINT64,	/* CPU time	*/
+				    G_TYPE_ULONG,	/* Started	*/
 				    G_TYPE_INT,		/* Nice		*/
 				    G_TYPE_UINT,	/* ID		*/
 				    G_TYPE_STRING,	/* Security Context */
@@ -342,18 +309,45 @@ proctable_new (ProcData * const procdata)
 
 		// type
 		switch (i) {
-		case COL_VMSIZE:
-		case COL_MEMRES:
-		case COL_MEMWRITABLE:
-		case COL_MEMSHARED:
 		case COL_MEMXSERVER:
-		case COL_MEM:
 		  gtk_tree_view_column_set_cell_data_func(col, cell,
 							  &procman::size_cell_data_func,
 							  GUINT_TO_POINTER(i),
 							  NULL);
 		  break;
 
+		case COL_VMSIZE:
+		case COL_MEMRES:
+		case COL_MEMSHARED:
+		case COL_MEM:
+		case COL_MEMWRITABLE:
+		  gtk_tree_view_column_set_cell_data_func(col, cell,
+							  &procman::size_na_cell_data_func,
+							  GUINT_TO_POINTER(i),
+							  NULL);
+		  break;
+
+		case COL_CPU_TIME:
+		  gtk_tree_view_column_set_cell_data_func(col, cell,
+							  &procman::duration_cell_data_func,
+							  GUINT_TO_POINTER(i),
+							  NULL);
+		  break;
+
+		case COL_START_TIME:
+		  gtk_tree_view_column_set_cell_data_func(col, cell,
+							  &procman::time_cell_data_func,
+							  GUINT_TO_POINTER(i),
+							  NULL);
+		  break;
+
+		case COL_STATUS:
+		  gtk_tree_view_column_set_cell_data_func(col, cell,
+							  &procman::status_cell_data_func,
+							  GUINT_TO_POINTER(i),
+							  NULL);
+		  break;
+
 		default:
 		  gtk_tree_view_column_set_attributes(col, cell, "text", i, NULL);
 		  break;
@@ -390,23 +384,6 @@ proctable_new (ProcData * const procdata)
 
 	gtk_container_add (GTK_CONTAINER (scrolled), proctree);
 
-
-	for(i = COL_NAME; i <= COL_MEM; i++)
-	{
-		switch(i)
-		{
-		case COL_CPU_TIME:
-		case COL_START_TIME:
-		case COL_CPU:
-			gtk_tree_sortable_set_sort_func (
-				GTK_TREE_SORTABLE (model),
-				i,
-				sort_ints,
-				GINT_TO_POINTER(i),
-				NULL);
-		}
-	}
-
 	procdata->tree = proctree;
 
 	set_proctree_reorderable(procdata);
@@ -444,35 +421,6 @@ ProcInfo::~ProcInfo()
 }
 
 
-static void
-get_process_status (ProcInfo *info, const glibtop_proc_state *buf)
-{
-
-	switch(buf->state)
-	{
-	case GLIBTOP_PROCESS_RUNNING:
-		info->status = _("Running");
-		break;
-
-	case GLIBTOP_PROCESS_STOPPED:
-		info->status = _("Stopped");
-		break;
-
-	case GLIBTOP_PROCESS_ZOMBIE:
-		info->status = _("Zombie");
-		break;
-
-	case GLIBTOP_PROCESS_UNINTERRUPTIBLE:
-		info->status = _("Uninterruptible");
-		break;
-
-	default:
-		info->status = _("Sleeping");
-		break;
-	}
-}
-
-
 static void
 get_process_name (ProcInfo *info,
 		  const gchar *cmd, const gchar *args)
@@ -565,50 +513,12 @@ get_process_memory_info(ProcInfo *info)
 
 	get_process_memory_writable(info);
 
-	info->mem = info->memxserver + info->memwritable;
+	// fake the smart memory column if writable is not available
+	info->mem = info->memxserver + (info->memwritable ? info->memwritable : info->memres);
 }
 
 
 
-static inline unsigned divide(unsigned *q, unsigned *r, unsigned d)
-{
-	*q = *r / d;
-	*r = *r % d;
-	return *q != 0;
-}
-
-/*
- * @param d: duration in centiseconds
- * @type d: unsigned
- */
-static char *
-format_duration_for_display(unsigned centiseconds)
-{
-	unsigned weeks = 0, days = 0, hours = 0, minutes = 0, seconds = 0;
-
-	(void)(divide(&seconds, &centiseconds, 100)
-	       && divide(&minutes, &seconds, 60)
-	       && divide(&hours, &minutes, 60)
-	       && divide(&days, &hours, 24)
-	       && divide(&weeks, &days, 7));
-
-	if (weeks)
-		/* xgettext: weeks, days */
-		return g_strdup_printf(_("%uw%ud"), weeks, days);
-
-	if (days)
-		/* xgettext: days, hours (0 -> 23) */
-		return g_strdup_printf(_("%ud%02uh"), days, hours);
-
-	if (hours)
-		/* xgettext: hours (0 -> 23), minutes, seconds */
-		return g_strdup_printf(_("%u:%02u:%02u"), hours, minutes, seconds);
-
-	/* xgettext: minutes, seconds, centiseconds */
-	return g_strdup_printf(_("%u:%02u.%02u"), minutes, seconds, centiseconds);
-}
-
-
 static void
 update_info_mutable_cols(ProcInfo *info)
 {
@@ -616,15 +526,6 @@ update_info_mutable_cols(ProcInfo *info)
 	GtkTreeModel *model;
 	model = gtk_tree_view_get_model(GTK_TREE_VIEW(procdata->tree));
 
-	gchar *cpu_time, *start_time;
-
-	// expects centiseconds
-	cpu_time = format_duration_for_display(100 * info->cpu_time_last / procdata->frequency);
-
-	/* FIXME: does it worths it to display relative to $now date ?
-	   absolute date wouldn't required to be updated on every refresh */
-	start_time = procman_format_date_for_display(info->start_time);
-
 	gtk_tree_store_set(GTK_TREE_STORE(model), &info->node,
 			    COL_STATUS, info->status,
 			    COL_USER, info->user,
@@ -634,16 +535,11 @@ update_info_mutable_cols(ProcInfo *info)
 			    COL_MEMSHARED, info->memshared,
 			    COL_MEMXSERVER, info->memxserver,
 			    COL_CPU, guint(info->pcpu),
-			    COL_CPU_TIME, cpu_time,
-			    COL_START_TIME, start_time,
+			    COL_CPU_TIME, info->cpu_time,
+			    COL_START_TIME, gulong(info->start_time),
 			    COL_NICE, gint(info->nice),
 			    COL_MEM, info->mem,
 			    -1);
-
-	/* FIXME: We don't bother updating COL_SECURITYCONTEXT as it can never change.
-	   even on fork ? */
-	g_free (cpu_time);
-	g_free(start_time);
 }
 
 
@@ -675,11 +571,6 @@ insert_info_to_tree (ProcInfo *info, ProcData *procdata, bool forced = false)
 	else
 		gtk_tree_store_insert (GTK_TREE_STORE (model), &info->node, NULL, 0);
 
-	/* COL_POINTER must be set first, because GtkTreeStore
-	 * will call sort_ints as soon as we set the column
-	 * that we're sorting on.
-	 */
-
 	gtk_tree_store_set (GTK_TREE_STORE (model), &info->node,
 			    COL_POINTER, info,
 			    COL_NAME, info->name,
@@ -743,22 +634,22 @@ update_info (ProcData *procdata, ProcInfo *info)
 	glibtop_proc_time proctime;
 
 	glibtop_get_proc_state (&procstate, info->pid);
+	info->status = procstate.state;
+
 	glibtop_get_proc_uid (&procuid, info->pid);
 	glibtop_get_proc_time (&proctime, info->pid);
 
-	get_process_status (info, &procstate);
-
 	get_process_memory_info(info);
 
 	get_process_user(procdata, info, procstate.uid);
 
-	info->pcpu = (proctime.rtime - info->cpu_time_last) * 100 / procdata->cpu_total_time;
+	info->pcpu = (proctime.rtime - info->cpu_time) * 100 / procdata->cpu_total_time;
 	info->pcpu = MIN(info->pcpu, 100);
 
 	if (procdata->config.solaris_mode)
 	  info->pcpu /= procdata->config.num_cpus;
 
-	ProcInfo::cpu_times[info->pid] = info->cpu_time_last = proctime.rtime;
+	ProcInfo::cpu_times[info->pid] = info->cpu_time = proctime.rtime;
 	info->nice = procuid.nice;
 	info->ppid = procuid.ppid;
 }
@@ -769,7 +660,7 @@ ProcInfo::ProcInfo(pid_t pid)
     name(NULL),
     user(NULL),
     arguments(NULL),
-    status(NULL),
+    status(0),
     security_context(NULL),
     pid(pid),
     uid(-1)
@@ -801,7 +692,7 @@ ProcInfo::ProcInfo(pid_t pid)
 	    if (proctime.rtime >= it->second)
 	      cpu_time = it->second;
 	  }
-	info->cpu_time_last = cpu_time;
+	info->cpu_time = cpu_time;
 	info->start_time = proctime.start_time;
 
 	get_process_selinux_context (info);
@@ -1048,6 +939,6 @@ ProcInfo::set_icon(Glib::RefPtr<Gdk::Pixbuf> icon)
   GtkTreeModel *model;
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(ProcData::get_instance()->tree));
   gtk_tree_store_set(GTK_TREE_STORE(model), &this->node,
-		     COL_PIXBUF, (this->pixbuf ? this->pixbuf->gobj() : NULL),
+		     COL_PIXBUF, this->pixbuf->gobj(),
 		     -1);
 }
